package org.openxma.dsl.common.formatter;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.xtext.AbstractElement;
import org.eclipse.xtext.parsetree.AbstractNode;
import org.openxma.dsl.common.formatter.locator.LineWrapLocator;
import org.openxma.dsl.common.formatter.locator.ComplexLocator;
import org.openxma.dsl.common.formatter.locator.SimpleLocator;

public class OpenXMAFormattingConfig {
	
	protected LocatorTree noSpaceLocators = new LocatorTree();
	protected LocatorTree startIndentionLocators = new LocatorTree();	
	protected LocatorTree endIndentionLocators = new LocatorTree();
	protected LocatorTree lineWrapLocators = new LocatorTree();		
				
	private String indentionSpace;
	private String indentionSpace2;
	
	public OpenXMAFormattingConfig() {
		setIndentationSpace("    ");
	}
	
	public SimpleLocator setLinewrap(int lines) {
		return new LineWrapLocator(lineWrapLocators,lines);		
	}
	
	public void setIndentationSpace(String space) {
		indentionSpace = space;
		indentionSpace2 = space+space;				
	}

	/**
	 * Configure that there is a indentation started before the first
	 * given element and that it ends before the second given element.
	 * 
	 * @param startIndention
	 * @param endIndention
	 */
	public void setIndentation(EObject startIndention,
			EObject endIndention) {		
		if (startIndention != null) {
			new SimpleLocator(startIndentionLocators).before(startIndention);
		}
		if (endIndention != null) {
			new SimpleLocator(endIndentionLocators).before(endIndention);
		}
	}
	
	/**
	 * Configure that the given element is intended. This means the indentation 
	 * starts before this element and ends after this element.
	 * 
	 * @param element
	 */
	public void setIndentation(EObject element) {
		startIndentation().before(element);		
		endIndentation().after(element);					
	}
	
	public SimpleLocator startIndentation() {
		return new SimpleLocator(startIndentionLocators);
	}
	
	public SimpleLocator endIndentation() {
		return new SimpleLocator(endIndentionLocators);
	}

    public String getIndention(int count) {
    	if (count < 0 ) {
    		throw new IllegalArgumentException("Count of indent must be greator or equal zero.");
    	}
    	if (count == 0) return "";
    	if (count == 1) return indentionSpace;
    	if (count == 2) return indentionSpace2;  	
    	StringBuffer buf = new StringBuffer(indentionSpace.length()*count);
    	buf.append(indentionSpace2); 	
    	for (int i=2;i<count;i++) {
    		buf.append(indentionSpace);
    	}
    	return buf.toString();
    }
    
	public boolean allowSpace(AbstractNode node, EObject elementOfLeftNode, EObject elementOfRightNode) {								
		if (noSpaceLocators.isLocatorDefined(node,elementOfLeftNode,elementOfRightNode)) {
			return false;
		}				
		return true;
	}
	
	public ComplexLocator setNoSpace() {
		return new ComplexLocator(noSpaceLocators);
	}				
}
